/* {{{ Copyright etc. */

/*  avwstats.c - basic image stats measurements

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwstats <input> [options]\n");
  printf("-l <lthresh> : set lower threshold\n");
  printf("-u <uthresh> : set upper threshold\n");
  printf("-r           : output <robust min intensity> <robust max intensity>\n");
  printf("-R           : output <min intensity> <max intensity>\n");
  printf("-v           : output <voxels> <volume>\n");
  printf("-m           : output mean\n");
  printf("Note - thresholds are not inclusive ie lthresh<allowed<uthresh\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc, argv)
  int   argc;
  char  *argv [];
{
  /* {{{ vars */

image_struct im;
int i;

/* }}} */

  if (argc<3)
    usage();

  avw_read(argv[1],&im);
  im.thresh=im.min=im.max=0; /* flag up that the thresholds need finding */

  for (i = 2; i < argc; i++) {
    if (!strcmp(argv[i], "-l"))
      /* {{{ set lower threshold */

{
  i++;
  if (argc<i+1)
    {
      printf("Error: no value given following -l\n");
      usage();
    }

  im.lthresh=atof(argv[i]);
  im.thresh=im.min=im.max=0; /* flag up that the thresholds need re-finding */
}

/* }}} */
    else if (!strcmp(argv[i], "-u"))
      /* {{{ set upper threshold */

{
  i++;
  if (argc<i+1)
    {
      printf("Error: no value given following -u\n");
      usage();
    }

  im.uthresh=atof(argv[i]);
  im.thresh=im.min=im.max=0; /* flag up that the thresholds need re-finding */
}

/* }}} */
    else if (!strcmp(argv[i], "-r"))
      /* {{{ robust range */

{
  if (im.thresh==im.min)
    find_thresholds(&im,0.1);

  printf("%f %f ",(double)im.thresh2,(double)im.thresh98);
}

/* }}} */
    else if (!strcmp(argv[i], "-R"))
      /* {{{ complete range */

{
  if (im.thresh==im.min)
    find_thresholds(&im,0.1);

  printf("%f %f ",(double)im.min,(double)im.max);
}

/* }}} */
    else if (!strcmp(argv[i], "-v"))
      /* {{{ voxels/volume between thresholds */

{
  int j, count=0;

  for(j=0;j<im.x*im.y*im.z*im.t;j++)
    if ( (im.i[j]>im.lthresh) && (im.i[j]<im.uthresh) )
      count++;

  printf("%d %f ",count,count*im.xv*im.yv*im.zv);
}

/* }}} */
    else if (!strcmp(argv[i], "-m"))
      /* {{{ mean */

{
  int j, count=0;
  float sum=0;

  for(j=0;j<im.x*im.y*im.z*im.t;j++)
    if ( (im.i[j]>im.lthresh) && (im.i[j]<im.uthresh) )
      {
	sum+=im.i[j];
	count++;
      }

  printf("%f ",sum/count);
}

/* }}} */
    else
      usage();
  }

  printf("\n");
  return(0);
}

/* }}} */
