/*  ols.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include <iostream>
#include <fstream>
#define WANT_STREAM
#define WANT_MATH

#include "newmatap.h"
#include "newmatio.h"

#if !defined(__ols_h)
#define __ols_h

#ifndef NO_NAMESPACE
using namespace NEWMAT;
namespace SIGPROC{
#endif
 
#define FALSE 0
#define TRUE 1

#define BATCHSIZE 50

  class Ols
    {
    public:
     
      Ols(const Matrix& p_y, const Matrix& p_x, const Matrix& p_contrasts); 

      const Matrix& ComputeResids();
      const ColumnVector& ComputeVar();
      const ColumnVector& Computecb();
      const float ComputeVar(const int ind);
      const float Computecb(const int ind);
      const ColumnVector& Computeb(const int ind);

      float SetupWithV(const ColumnVector& p_vrow, bool p_justvarone = FALSE);
      float SetupWithV(const ColumnVector& p_vrow, const ColumnVector& p_krowspec, ColumnVector& p_res, bool p_justvarone = FALSE, const int zeropad = 512);

      const Matrix& Getr() const {return r;}
     
      const ColumnVector& Getvar() const {return var;}
      const ColumnVector& Getcb() const {return cb;}
      float Getdof() const {return dof;}
      float Getvarone() const {return var_on_e;}

      void SetContrast(const int p_num) {c = contrasts.Row(p_num).t();}
      void Setvarone(const float p_varone) {var_on_e = p_varone;}
      
    private:
      Ols();
      Ols(const Ols&);
      Ols& operator=(const Ols& p_ols);
       
      void ConstructV(const ColumnVector& p_vrow);

      // y = bx + r
      const Matrix& y;
      const Matrix& x;
   
      // Contrast
      const Matrix& contrasts;
      ColumnVector c;

      int numTS;
      int sizeTS;

      Matrix r;

      Matrix pinv_x;
      float var_on_e;

      ColumnVector cb;
      
      ColumnVector b;

      ColumnVector var;
      float dof;

      // Covariance matrix is sigma^2*V:
      Matrix V;
      Matrix RV;

      Matrix RMat;
      
      // batch stuff
      int batch_size;
    };

#endif

#ifndef NO_NAMESPACE
       }
#endif



