#ifndef __GLOBALOPTIONS_
#define __GLOBALOPTIONS_

#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include "generalio.h"
#include "costfns.h"

using namespace COSTFNS;
using namespace GENERALIO;

enum anglereps { Euler, Quaternion };

class Globaloptions {
 public:
  static Globaloptions& getInstance();
  ~Globaloptions() { delete gopt; }
  
  string inputfname;
  string outputfname;
  int verbose;
  int no_params;
  int dof;
  int no_bins;
  int refnum;
  int no_volumes;
  int gdtflag;
  int edgeflag;
  int histflag;
  int matflag;
  int statflag;
  int plotflag;
  int tmpmatflag;
  int no_reporting;
  float scaling;
  short datatype;
  float smoothsize;
  
  costfns maincostfn;
  imagepair *impair;  
  Matrix initmat;
  anglereps anglerep;
  ColumnVector boundguess;

  void parse_command_line(int argc, char** argv);

 private:
  Globaloptions();
  
  const Globaloptions& operator=(Globaloptions&);
  Globaloptions(Globaloptions&);
      
  static Globaloptions* gopt;

  void print_usage(int argc, char *argv[]);
  
};

inline Globaloptions& Globaloptions::getInstance(){
  if(gopt == NULL)
    gopt = new Globaloptions();
  
  return *gopt;
}

inline Globaloptions::Globaloptions()
{
  outputfname = "";
  inputfname = "";
  no_params = 6;
  dof = 6;
  no_bins = 256;
  anglerep = Euler;
  verbose = 0;
  maincostfn = NormCorr;
  initmat = Identity(4);
  impair = 0;
  refnum = -1;
  no_volumes = 0;
  gdtflag = 0;
  edgeflag = 0;
  histflag = 0;
  matflag = 0;
  statflag = 0;
  plotflag = 0;
  tmpmatflag = 0;
  no_reporting = 1;
  scaling = 6.0;
  smoothsize = 1.0;
  boundguess.ReSize(2);
  boundguess << 10.0 << 1.0;

}

#endif

