#   fsl - meta-menu for fsl tools
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

#{{{ setups

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

#}}}

proc fsl { w } {

    #{{{ vars and setup

global FSLDIR INMEDX argc argv PWD HOME

toplevel $w -menu $w.f1.fmrib.menu
wm title $w "FSL"
wm iconname $w "FSL"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

set graphpic [image create photo -file ${FSLDIR}/doc/images/fsl-logo_small.gif ]

frame $w.f1

button $w.f1.label -image $graphpic -borderwidth 0 \
	-command "FmribWebHelp file: ${FSLDIR}/doc/index.html" \

set BW 19

#}}}
    #{{{ main menu

pack $w.f1.label -in $w.f1

if { [ file exists ${FSLDIR}/tcl/loadvarian.tcl ] } {
    button $w.f1.loadvarian -text "Load Varian" -width $BW \
	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/LoadVarian" &} junk]}
    pack $w.f1.loadvarian -in $w.f1
}

if { [ file exists ${FSLDIR}/tcl/bet.tcl ] } {
    button $w.f1.bet -text "BET Brain Extraction" -width $BW \
	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Bet" &} junk]}
    pack $w.f1.bet -in $w.f1
}

if { [ file exists ${FSLDIR}/tcl/flirt.tcl ] } {
    button $w.f1.flirt -text "FLIRT Linear Registration" -width $BW \
	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Flirt" &} junk]}
    pack $w.f1.flirt -in $w.f1
}

if { [ file exists ${FSLDIR}/tcl/susan.tcl ] } {
    button $w.f1.susan -text "SUSAN Noise Reduction" -width $BW \
	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Susan" &} junk]}
    pack $w.f1.susan -in $w.f1
}

if { [ file exists ${FSLDIR}/tcl/feat.tcl ] } {
    button $w.f1.feat -text "FEAT" -width $BW \
	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Feat" &} junk]}
    pack $w.f1.feat -in $w.f1
}


#}}}
    #{{{ COMMENT general menu

#menubutton $w.f1.general -text "General" -width $BW -menu $w.f1.general.menu
#
#menu $w.f1.general.menu
#
#if { [ file exists ${FSLDIR}/tcl/bet.tcl ] } {
#    $w.f1.general.menu add command -label "BET Brain Extraction" \
#	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Bet" &} junk]}
#}
#
#if { [ file exists ${FSLDIR}/tcl/flirt.tcl ] } {
#    $w.f1.general.menu add command -label "FLIRT Linear Registration" \
#	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Flirt" &} junk]}
#}
#
#if { [ file exists ${FSLDIR}/tcl/susan.tcl ] } {
#    $w.f1.general.menu add command -label "SUSAN Nonlinear Noise Reduction" \
#	    -command {set status [catch {exec sh -ce "${FSLDIR}/bin/Susan" &} junk]}
#}
#

#}}}
    #{{{ Button Frame

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
 
    button $w.cancel    -command "destroy $w" \
        -text "Exit" -width 5
    bind $w.cancel <Return> {
        [winfo toplevel %W].cancel invoke}
 
    button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/fslmenu.html" \
            -text "Help" -width 5
    bind $w.help <Return> {
        [winfo toplevel %W].help invoke}

    pack $w.btns.b -side bottom -fill x
    pack $w.cancel $w.help -in $w.btns.b \
        -side left -expand yes -padx 3 -pady 3 -fill y
 
    pack $w.f1 $w.btns -expand yes -fill both

#}}}
}

wm withdraw .
fsl .rename
tkwait window .rename
