#

#   register - view registration of two images
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc mni_register {Folder Group TheArgs} {
    global PXHOME USER PROCID exceed

# find input images
MxGetPagesFromGroup $Group Pages
set Image1 [ lindex $Pages 0 ]
set Image2 [ lindex $Pages 1 ]

# save image to temp AVW file
ScriptUpdate "Saving temporary files"
FSLSaveAs $Image1 AVW /tmp/${USER}_${PROCID}_image1.hdr true
FSLSaveAs $Image2 AVW /tmp/${USER}_${PROCID}_image2.hdr true

# convert to minc
set thecommand " exec /usr/local/mni/bin/analyze2minc /tmp/${USER}_${PROCID}_image1 "
ScriptUpdate "Converting to MINC format: $thecommand"
set result [catch {eval $thecommand} ErrMsg]
set thecommand " exec /usr/local/mni/bin/analyze2minc /tmp/${USER}_${PROCID}_image2 "
ScriptUpdate "Converting to MINC format: $thecommand"
set result [catch {eval $thecommand} ErrMsg]
CancelScriptUpdate

# run external program
set thecommand "exec xterm -e /usr/local/mni/bin/register /tmp/${USER}_${PROCID}_image1.mnc /tmp/${USER}_${PROCID}_image2.mnc &"
set result [catch {eval $thecommand} ErrMsg]

after 1800000

set errrr [catch {exec /bin/rm -f /tmp/${USER}_${PROCID}_image1.mnc /tmp/${USER}_${PROCID}_image1.img /tmp/${USER}_${PROCID}_image1.hdr /tmp/${USER}_${PROCID}_image2.mnc /tmp/${USER}_${PROCID}_image2.img /tmp/${USER}_${PROCID}_image2.hdr } junk]
set errrr [catch {exec /bin/rm -f /tmp/${USER}_${PROCID}_image1.mnc /tmp/${USER}_${PROCID}_image1.img /tmp/${USER}_${PROCID}_image1.hdr /tmp/${USER}_${PROCID}_image2.mnc /tmp/${USER}_${PROCID}_image2.img /tmp/${USER}_${PROCID}_image2.hdr } junk]
set errrr [catch {exec /bin/rm -f /tmp/${USER}_${PROCID}_image1.mnc /tmp/${USER}_${PROCID}_image1.img /tmp/${USER}_${PROCID}_image1.hdr /tmp/${USER}_${PROCID}_image2.mnc /tmp/${USER}_${PROCID}_image2.img /tmp/${USER}_${PROCID}_image2.hdr } junk]

}
