#

#{{{ setup

#   renderstats - colour rendering of two or more images
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

#}}}

#{{{ proc render

proc render { w } {
    global rendervars PXHOME FSLDIR
 
    #{{{ setup main window

toplevel $w

wm title $w "Statistics Colour Rendering"
wm iconname $w "Render"
wm iconbitmap $w @$FSLDIR/tcl/fmrib.xbm

frame $w.f

#}}}
    #{{{ Stats

set rendervars($w,maxstatss) 2
set rendervars($w,statss) 1

tixControl $w.f.statss -label " Number of stats images " \
	-variable rendervars($w,statss) -step 1 -min 1 -max $rendervars($w,maxstatss) -selectmode immediate -command "render:updatestats $w"

pack $w.f.statss -in $w.f -anchor w -pady 5

set i 1
while { $i <= $rendervars($w,maxstatss) } {
    set rendervars($w,statsmax,$i) "max"
    frame $w.f.statsentries($i)
    label $w.f.uel($i) -width 18 -text "Stats image ${i} "
    entry $w.f.uee($i) -textvariable rendervars($w,statsentries,$i) -width 35
    label $w.f.uelmin($i) -text " Min"
    entry $w.f.uemin($i) -textvariable rendervars($w,statsmin,$i) -width 7
    label $w.f.uelmax($i) -text " Max"
    entry $w.f.uemax($i) -textvariable rendervars($w,statsmax,$i) -width 7
    pack $w.f.uel($i) $w.f.uee($i) $w.f.uelmin($i) $w.f.uemin($i) $w.f.uelmax($i) $w.f.uemax($i) -in $w.f.statsentries($i) -padx 3 -pady 3 -side left
    incr i 1
}

render:updatestats $w 1

$w.f.uelmin(1) configure -fg "red"
$w.f.uelmax(1) configure -fg "yellow"
$w.f.uelmin(2) configure -fg "blue"
$w.f.uelmax(2) configure -fg "cyan"

#}}}
    #{{{ Background image

set rendervars($w,statsmin,0) "min"
set rendervars($w,statsmax,0) "max"

frame $w.f.statsentries(0)
label $w.f.uel(0) -width 18 -text "Background image "
entry $w.f.uee(0) -textvariable rendervars($w,statsentries,0) -width 35
label $w.f.uelmin(0) -text " Min"
entry $w.f.uemin(0) -textvariable rendervars($w,statsmin,0) -width 7
label $w.f.uelmax(0) -text " Max"
entry $w.f.uemax(0) -textvariable rendervars($w,statsmax,0) -width 7
pack $w.f.uel(0) $w.f.uee(0) $w.f.uelmin(0) $w.f.uemin(0) $w.f.uelmax(0) $w.f.uemax(0) -in $w.f.statsentries(0) -padx 3 -pady 3 -side left

$w.f.uelmax(0) configure -fg "white"

#}}}
    #{{{ Colourmap type

frame $w.f.types

tixOptionMenu $w.f.type -label " Colourmap type:  " -variable rendervars($w,type)
$w.f.type add command 0 -label "Solid colours"
$w.f.type add command 1 -label "Transparent colours"
set rendervars($w,type) 1

set rendervars($w,checker) 0
label $w.f.checkerlabel -text " Apply checkerboard mask to colour overlay"
checkbutton $w.f.checker -variable rendervars($w,checker)

pack $w.f.type $w.f.checkerlabel $w.f.checker -in $w.f.types -padx 3 -pady 3 -side left

#}}}
    #{{{ Image type:

set rendervars($w,output) 0

tixOptionMenu $w.f.output -label " Rendered image type:  " \
        -variable rendervars($w,output)

$w.f.output add command 0 -label "Floating point; red-yellow blobs contain original stats values if solid colours are used"
$w.f.output add command 1 -label "Integer; needed for later 3D rendering"

pack $w.f.output $w.f.types $w.f.statsentries(0) -anchor w -side bottom

#}}}
    #{{{ button frame

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
    
    button $w.ok -text "OK" -width 5 -command "render:apply $w destroy"
    bind $w.ok <Return> {
        [winfo toplevel %W].ok invoke
    }
    
    button $w.apply -command "render:apply $w keep" -text "Apply" -width 5
    bind $w.apply <Return> {
	[winfo toplevel %W].apply invoke
    }
	    
    set rendervars($w,cmap) 0
    button $w.loadcmap    -command "set rendervars($w,cmap) 1; render:apply $w keep; set rendervars($w,cmap) 0" -text "Load Colour LUT"

    button $w.cancel    -command "render:destroy $w" \
	    -text "Cancel" -width 5
    bind $w.cancel <Return> {
	[winfo toplevel %W].cancel invoke
    }

    button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/renderstats/index.html" \
            -text "Help" -width 5
    bind $w.help <Return> {
        [winfo toplevel %W].help invoke
    }

    pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
    pack $w.ok $w.apply $w.loadcmap $w.cancel $w.help -in $w.btns.b \
	    -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
		
    pack $w.f $w.btns -expand yes -fill both
}

#}}}
#{{{ proc render:apply

proc render:apply { w dialog } {
    global rendervars PXHOME

    MxGetCurrentFolder Folder
    set statslist ""

    if { ! $rendervars($w,cmap) } {
	#{{{ process entries

set i 0
while { $i <= $rendervars($w,statss) } {
    
    set success [ MxGetPageByName $Folder $rendervars($w,statsentries,$i) statspage($i) ]

    if { $success } {
	MxPause "Not all of the images are valid"
	return
    }

    set depth($i) 1
    MxGetDimensions $statspage($i) width($i) height($i) depth($i)
    MxGetImageProperties $statspage($i) pl
    set bpp($i) [ keylget pl Bits ]
    set enc($i) [ keylget pl Encoding ]

    if { $i > 0 } {
	if { $depth($i) != $depth(0) || $width($i) != $width(0) || $height($i) != $height(0) } {
	    MxPause "Not all of the STATS images are of the same size and type as the input image"
	    return
	}
    }


    lappend statslist $statspage($i) 

    if { $rendervars($w,statsmin,$i) == "" || $rendervars($w,statsmax,$i) == "" } {
	MxPause "Not all of the thresholds have been set"
	return
    }

    MxQuietStatistics $statspage($i) { Minimum Maximum } false Stats
    set themin [ keylget Stats Minimum ]
    set themax [ keylget Stats Maximum ]

    if { $rendervars($w,statsmin,$i) == "min" } {
	set rendervars($w,statsmin,$i) $themin
    }
    if { $rendervars($w,statsmax,$i) == "max" } {
	set rendervars($w,statsmax,$i) $themax
    }

    lappend statslist $rendervars($w,statsmin,$i)
    lappend statslist $rendervars($w,statsmax,$i)

    incr i 1
}

puts " $statslist "

#}}}
    }
    set rendervars($w,cmap) 0

    render_proc $Folder $rendervars($w,type) $rendervars($w,output) $rendervars($w,checker) $rendervars($w,statss) $statslist
    
    update idletasks
    
    if {$dialog == "destroy"} {
	render:destroy $w
    }
}

#}}}
#{{{ proc render:destroy

proc render:destroy { w } {
    destroy $w
}

#}}}
#{{{ updates

proc render:updatestats { w val } {
    global rendervars

    set i 1
    while { $i <= $rendervars($w,maxstatss) } {
	pack forget $w.f.statsentries($i)
	incr i 1
    }

    set i 1
    while { $i <= $rendervars($w,statss) } {
	pack $w.f.statsentries($i) -in $w.f -anchor w -pady 5
	incr i 1
    }
}

#}}}

#{{{ tail end

render .rename
tkwait window .rename

#}}}
