/*  Log.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#if !defined(Log_h)
#define Log_h

#include <iostream>
#include <fstream>
#include <string>
#include "newmatap.h"
#include "newmatio.h"

using namespace NEWMAT;

namespace Utilities{

  template<class t> string tostring(const t obj);
  
  class Log
    {
    public:
      static Log& getInstance();
      ~Log() { delete logger; }

      /** Makes a directory to place results into:
	  keeps adding "+" to pdirname until unique directory is made: */
      void makeDir(const string& pdirname, const string& plogfilename = "logfile");

      /** Sets an existing  directory to place results into: */       
      void setDir(const string& pdirname, const string& plogfilename = "logfile"); 
      
      const string& getDir() const { return dir; }
      const string& getLogFileName() const { return logfilename; }
      const string appendDir(const string& filename) const { return dir + "/" + filename;}

      void out(const string& p_fname, const Matrix& p_mat);
      void out(const string& p_fname, const RowVector& p_mat);
      void out(const string& p_fname, const ColumnVector& p_mat);

      static int counter() { return count++; }
      ofstream& str();
      
    private:
      Log():logEstablished(false) {}
      
      const Log& operator=(Log&);
      Log(Log&);
      
      static Log* logger;
      string dir;
      ofstream logfileout;
      string logfilename;
      static int count;

      bool logEstablished;
    };
  
  inline void Log::out(const string& p_fname, const Matrix& p_mat)
    {

      if(!logEstablished)
	{
	  logfilename = "logfile";
	  setDir(".");
	}

      ofstream out;
      out.open((dir + "/" + p_fname).c_str(), ios::out);
      out.setf(ios::scientific, ios::floatfield);

      for(int i=1; i<=p_mat.Nrows(); i++)	   
	{
	  for(int j=1; j<=p_mat.Ncols(); j++)
	    {
	      out << p_mat(i,j) << " ";	
	    }
	  out << endl;
	}

      out.close();
    }
  
  inline void Log::out(const string& p_fname, const ColumnVector& p_mat)
    {
      if(!logEstablished)
	{
	  logfilename = "logfile";
	  setDir(".");
	}

      ofstream out;
      out.open((dir + "/" + p_fname).c_str(), ios::out);     
      out.setf(ios::scientific, ios::floatfield);
      for(int j=1; j<=p_mat.Nrows(); j++)
	{
	  out << p_mat(j);	  
	  out << endl;
	}
      
      out.close();
    }

  inline void Log::out(const string& p_fname, const RowVector& p_mat)
    {

      if(!logEstablished)
	{
	  logfilename = "logfile";
	  setDir(".");
	}

      ofstream out;
      out.open((dir + "/" + p_fname).c_str(), ios::out);
      out.setf(ios::scientific, ios::floatfield);

      for(int j=1; j<=p_mat.Ncols(); j++)
	{
	  out << p_mat(j) << " ";
	}
      out << endl;
      out.close();
    }

  inline Log& Log::getInstance(){
    if(logger == NULL)
      logger = new Log();
  
    return *logger;
  }

  inline ofstream& Log::str() { 
    
    if(!logEstablished)
      {
	logfilename = "logfile";
	setDir(".");
      }

    return logfileout; 
  }
 
}

#endif

