#{{{ copyright and setup

#   FEAT - FMRIB's Easy Analysis Tool for FMRI
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

#}}}
#{{{ feat4:load

proc feat4:load { w } {
    global PXHOME FSLDIR VARS argc argv PWD entries loadtimeseries
 
    #{{{ setup main window

toplevel $w

wm title      $w "Load FEAT results into MEDx"
wm iconname   $w "FEAT Load"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

frame $w.f
pack $w.f -in $w -side top

FSLFileEntry $w.f.featdir \
	-variable entries($w,1) \
	-pattern "*.feat" \
	-directory "~" \
	-label "Select a FEAT directory" \
	-title "Select a FEAT directory" \
	-width 20 \
	-dirasfile "design.fsf" \
	-filterhist VARS(history)

frame $w.f.timeseries
label $w.f.timeseries.label -text "Load timeseries data"
set loadtimeseries 1
checkbutton $w.f.timeseries.yn -variable loadtimeseries
pack $w.f.timeseries.label $w.f.timeseries.yn -in $w.f.timeseries -side left -anchor w

pack $w.f.featdir $w.f.timeseries -in $w.f -padx 5 -pady 5 -anchor w

#}}}
    #{{{ Button Frame

frame $w.btns
frame $w.btns.b -relief raised -borderwidth 1
    
button $w.apply -command "feat4:load_proc $w" -text "Go"

button $w.cancel -command "destroy $w" -text "Exit"

pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
pack $w.apply $w.cancel -in $w.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

pack $w.btns -in $w -side bottom -fill x

#}}}
}

#}}}
#{{{ feat4:load_proc

proc feat4:load_proc { w } {
    #{{{ setup

    global PXHOME FSLDIR entries loadtimeseries

    set FD $entries($w,1)

    puts "Loading $FD"

    MxNewFolder folder window

    MxSetFolderProperties $folder "{Name $FD}"

if { ! [ file exists ${FD}/filtered_func_data.hdr ] } {
    set loadtimeseries 0
}

#}}}
    #{{{ load lowres files

catch { exec sh -c "echo ${FD}/example_func.hdr ${FD}/mask.hdr ${FD}/stats/zstat*hdr ${FD}/stats/fixed*hdr ${FD}/stats/random*hdr ${FD}/cluster_mask_*hdr ${FD}/thresh_*hdr ${FD}/Rendered_thresh_*.hdr" } lowreslist

set lowrespagelist ""
foreach lowres $lowreslist {
    if { [ file exists $lowres ] } {
	MxOpenImage $folder $lowres lowrespage
	set lowrespagelist "$lowrespage $lowrespagelist"
    }
}

#}}}
    if { $loadtimeseries } {
	#{{{ load func data

    MxOpenImage $folder ${FD}/filtered_func_data.hdr Group

    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]

    set AList ""
    lappend AList [ lindex $GPages 0 ]
    lappend AList [ lindex $GPages 1 ]
    MxGroupVolumes $AList "A" A

    set BList ""
    set i 2
    while { $i < $total_volumes } {
	lappend BList [ lindex $GPages $i ]
	incr i 1
    }
    MxGroupVolumes $BList "B" B

    MxBetweenGrpStatistics $A $B U blah blah2 blah3 FinalZscore ResultGroup

    MxGetPagesFromGroup $ResultGroup RP
    set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
    MxDeletePage { $RPList }
    MxDeleteGroupPages { $A $B $ResultGroup } No

    MxSetTimeSeriesProperties $Group 1 red

#}}}
    }
    #{{{ process lowres files

foreach lowrespage $lowrespagelist {
    MxSetCurrentPage $lowrespage
    MxGetImageProperties $lowrespage props
    if { $loadtimeseries } {
	MxDupPage $FinalZscore zscore
	MxExpr $zscore = " $lowrespage "
	MxDeletePage $lowrespage
    } else {
	set zscore $lowrespage
    }
    MxComputeDisplayRange $zscore V Min Max
    set thefile [ file rootname [ keylget props File ] ]
    set thename [ keylget props Name ]
    if { $loadtimeseries } {
	name_page $zscore $thename
    }

    if { [ string first Rendered $thename ] == 0 } {
	set cal_min  [ exec $FSLDIR/bin/avwval $thefile cal_min ]
	set cal_max  [ exec $FSLDIR/bin/avwval $thefile cal_max ]
	set aux_file [ string trimright [ exec $FSLDIR/bin/avwval $thefile aux_file ] " " ]
	MxSetDisplayRange $zscore V $cal_min $cal_max
	MxLoadMaterialMap $zscore ${FSLDIR}/etc/luts/${aux_file}.map
	MxLoadLUT $zscore ${FSLDIR}/etc/luts/${aux_file}.lut
    }
}

if { $loadtimeseries } {
    MxDeletePage $FinalZscore
}

#}}}
}

#}}}
#{{{ call GUI and wait

wm withdraw .
feat4:load .r
tkwait window .r

#}}}
